$debug
c
c     Automatic interpretation of vertical magnetic gradient
c        measurements  -  CONTACT solutions
c
      implicit real*8 (a-h,o-z)
      integer ispan(10)
      real*8 x(25),z(25),vg(800),amag(800)
     &,xl(800),pos(800),depth(800),dip(800)
      common/param/d,t,dep,alpha,susc,phi,xncl,decl,theta,h,rad,iok
c
c     contact solution - npt = 6
c
      npt=6
      rad=1./57.29578
c
c     h    = total field magnitude (nT)
c     ainc =   "     "   inclination (degrees) in plane of profile
c     str  = angle between profile strike and magnetic north (degrees)
c     isp  = number of spans used
c     tol = tolerance distance (metres)
c     ispan(i) = span used for run i
c
      read(1,*) h,ainc,str
      read(1,*)isp,tol
      read(1,*)(ispan(i),i=1,isp)
  50  format(5f10.3)
      write(2,8)
   8  format('0   totf       inc      str     tol ')
      write(2,50)h,ainc,str,tol
      xncl=ainc*rad
      decl=str*rad
      if(ainc.gt.89.5)then
      phi=xncl
      else
      phi=datan(dtan(xncl)/dcos(decl))
      endif
60    ktot=1
61    read (1,*,end=199) xl(ktot),vg(ktot)
      ktot=ktot+1
      go to 61
199   do100is=1,isp
      iok=0
      do4i=1,ktot-1-ispan(is)*(npt-1)
c solve for npt window
      do5j=1,npt
      x(j)=xl(i+(j-1)*ispan(is))
  5   z(j)=vg(i+(j-1)*ispan(is))
      call contac(x,z,npt)
c reject solutions if depth < 0
      if(dep.gt.0.0)then
c store acceptable solutions
      depth(iok)=dep
      dip(iok)=theta
      pos(iok)=d
      amag(iok)=susc
      endif
   4  continue
c keep successive solutions if distance in x and z directions
c less than distance TOL. Setting TOL equal to average data spacing
c usually works.
      k=1
      inew=1
      xp=pos(1)
      yp=depth(1)
      do9j=2,iok
      xx=pos(j)
      yy=depth(j)
      if(dabs(xx-xp).lt.tol.and.dabs(yy-yp).lt.tol)then
        if(inew.eq.1)then
          pos(k)=xp
          depth(k)=yp
          dip(k)=dip(j-1)
          amag(k)=amag(j-1)
          k=k+1
          inew=0
        endif
        pos(k)=xx
        depth(k)=yy
        dip(k)=dip(j)
        amag(k)=amag(j)
        k=k+1
      else
        inew=1
      endif
      xp=xx
  9   yp=yy
      k=k-1
      write(2,62)k,ispan(is)
  62  format(' # of solutions ',i5,' for span',i3)
      write(2,63)(pos(i),depth(i),dip(i),amag(i),i=1,k)
  63  format(3f12.3,g12.3)
 100  continue
 999  stop
      end
      subroutine contac(x,z,n)
c solve for contact parameters from npts window
      implicit real*8 (a-h,o-z)
      integer indx(6)
      real*8 x(6),z(6),a(6,6),c(6)
      common/param/d,t,dep,alpha,susc,phi,xncl,decl,theta,h,rad,iok
c reference x values to this point. Using true x values results
c in poor conditioning of matrix equations.
      x3=x(3)
      do77i=1,n
   77 x(i)=x(i)-x3
c set up matrix equations (23)
c figure in parenthesis refer to equations in Keating and Pilkington (1990)
      do5i=1,n
      a(i,1)=-z(i)
      a(i,2)=z(i)*x(i)
      a(i,3)=-x(i)**3
      a(i,4)=x(i)**2
      a(i,5)=x(i)
      a(i,6)=1.0e0
  5   c(i)=z(i)*x(i)**2
      np=6
c solve for coefficients e's
      call ludcmp(a,n,np,indx,x)
      call lubksb(a,n,np,indx,c)
c (24)
      d=c(2)/2.
c (25)
      dep=(c(1)-d**2)
      if(dep.lt.0.0)return
      dep=dsqrt(dep)
c (31)
      grad=c(3)
c (32)
      alev=grad*3.*d-c(4)
c (33)
      f=c(5)+dep**2*grad-2.*d*alev-3.*grad*d**2
c (33)
      ff=c(6)+d*f-dep**2*grad*d+dep**2*alev+grad*d**3+alev*d**2
      cota=ff/dep/f
      alp=datan(1./cota)
      if(alp.lt.0.0)alp=-alp
      thet=phi*2.-alp
      theta=thet/rad
c (33)
      amag=f/2./dsin(alp)
      aj=amag/dsin(thet)/((dsin(xncl)/dsin(phi))**2)
      susc=aj/h
c add back reference distance
      d=d+x3
      iok=iok+1
      return
      end
      subroutine ludcmp(a,n,np,indx,d)
      implicit real*8 (a-h,o-z)
      parameter (nmax=100,tiny=1.0d-20)
      dimension a(np,np),indx(n),vv(nmax)
      d=1.
      do 12 i=1,n
        aamax=0.
        do 11 j=1,n
          if (dabs(a(i,j)).gt.aamax) aamax=dabs(a(i,j))
11      continue
        if (aamax.eq.0.) pause 'singular matrix.'
        vv(i)=1./aamax
12    continue
      do 19 j=1,n
        if (j.gt.1) then
          do 14 i=1,j-1
            sum=a(i,j)
            if (i.gt.1)then
              do 13 k=1,i-1
                sum=sum-a(i,k)*a(k,j)
13            continue
              a(i,j)=sum
            endif
14        continue
        endif
        aamax=0.
        do 16 i=j,n
          sum=a(i,j)
          if (j.gt.1)then
            do 15 k=1,j-1
              sum=sum-a(i,k)*a(k,j)
15          continue
            a(i,j)=sum
          endif
          dum=vv(i)*dabs(sum)
          if (dum.ge.aamax) then
            imax=i
            aamax=dum
          endif
16      continue
        if (j.ne.imax)then
          do 17 k=1,n
            dum=a(imax,k)
            a(imax,k)=a(j,k)
            a(j,k)=dum
17        continue
          d=-d
          vv(imax)=vv(j)
        endif
        indx(j)=imax
        if(j.ne.n)then
          if(a(j,j).eq.0.)a(j,j)=tiny
          dum=1./a(j,j)
          do 18 i=j+1,n
            a(i,j)=a(i,j)*dum
18        continue
        endif
19    continue
      if(a(n,n).eq.0.)a(n,n)=tiny
      return
      end
      subroutine lubksb(a,n,np,indx,b)
      implicit real*8 (a-h,o-z)
      dimension a(np,np),indx(n),b(n)
      ii=0
      do 12 i=1,n
        ll=indx(i)
        sum=b(ll)
        b(ll)=b(i)
        if (ii.ne.0)then
          do 11 j=ii,i-1
            sum=sum-a(i,j)*b(j)
11        continue
        else if (sum.ne.0.) then
          ii=i
        endif
        b(i)=sum
12    continue
      do 14 i=n,1,-1
        sum=b(i)
        if(i.lt.n)then
          do 13 j=i+1,n
            sum=sum-a(i,j)*b(j)
13        continue
        endif
        b(i)=sum/a(i,i)
14    continue
      return
      end

